# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoAccount_Default_Config.ipynb.

# %% auto 0
__all__ = ['DomoAccount_Config', 'AccountConfig_UsesOauth', 'DomoAccount_NoConfig_OAuth', 'AccountConfig_ProviderTypeNotDefined',
           'DomoAccount_NoConfig', 'DomoAccount_Config_AbstractCredential', 'DomoAccount_Config_DatasetCopy',
           'DomoAccount_Config_DomoAccessToken', 'DomoAccount_Config_Governance', 'DomoAccount_Config_AmazonS3',
           'DomoAccount_Config_AmazonS3Advanced', 'DomoAccount_Config_AwsAthena',
           'DomoAccount_Config_HighBandwidthConnector', 'DomoAccount_Config_Snowflake',
           'DomoAccount_Config_SnowflakeUnload_V2', 'DomoAccount_Config_SnowflakeUnloadAdvancedPartition',
           'DomoAccount_Config_SnowflakeWriteback', 'DomoAccount_Config_SnowflakeUnload',
           'DomoAccount_Config_SnowflakeFederated', 'DomoAccount_Config_SnowflakeInternalUnload',
           'DomoAccount_Config_SnowflakeKeyPairAuthentication', 'AccountConfig']

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 3
from enum import Enum as ConfigEnum


# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 4
from copy import deepcopy

from dataclasses import dataclass, field
from abc import ABC, abstractmethod
import domolibrary.client.DomoError as dmde

from nbdev.showdoc import patch_to

import domolibrary.routes.account as account_routes
import domolibrary.client.DomoError as dmde
from typing import Any

from nbdev.showdoc import patch_to

import domolibrary.utils.DictDot as util_dd

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 7
@dataclass
class DomoAccount_Config:
    """DomoAccount Config abstract base class"""

    data_provider_type: str
    is_oauth : bool

    raw: dict = field(repr = False, default = None)
    parent: Any = field(repr = False, default = None)

    def __post_init__(self):
        if hasattr(self, 'raise_exception'):
            self.raise_exception()

    @classmethod
    @abstractmethod
    def _from_json(cls, obj: dict, data_provider_type: str = None, **kwargs):
        """convert accounts API response into a class object"""
        pass

    @abstractmethod
    def to_json(self):
        """convert class object into a format the accounts API expects"""
        pass

    def to_bucket(self) -> dict:
        keys = ["bucket", "secret_key", "access_key"]
        return {key: getattr(self, key) for key in keys}

    def to_user(self) -> dict:
        keys = ["account", "username", "password", "role", "data_provider_type"]
        return {key: getattr(self, key) for key in keys}

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 8
class AccountConfig_UsesOauth(dmde.DomoError):
    def __init__(self, data_provider_type):
        super().__init__(
            message=f"data provider type {data_provider_type} uses OAuth and therefore wouldn't return a Config object"
        )


@dataclass
class DomoAccount_NoConfig_OAuth(DomoAccount_Config):

    data_provider_type :str = None
    is_oauth : bool= True

    @classmethod
    def _from_json(cls, 
                   obj: dict, 
                   data_provider_type: str, 
                   parent: Any = None,
                   **kwargs):
        
        return cls(data_provider_type=data_provider_type, 
                   raw=obj,
                   parent = parent,
                   )

    def to_json(self):
        return {}
    
    def raise_exception(self):
        raise AccountConfig_UsesOauth(data_provider_type= self.data_provider_type)

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 10
class AccountConfig_ProviderTypeNotDefined(dmde.DomoError):
    def __init__(self, data_provider_type):
        super().__init__(
            message=f"data provider type {data_provider_type} not defined yet. Extend the AccountConfig class"
        )


@dataclass
class DomoAccount_NoConfig(DomoAccount_Config):
    is_oauth: bool = False

    @classmethod
    def _from_json(
        cls, 
        data_provider_type: str, 
        obj: dict, 
        parent: Any = None, 
        **kwargs
    ):

        return cls(
            data_provider_type=data_provider_type, parent=parent, raw=obj
        )

    def to_json(self):
        return {}

    def raise_exception(self):
        raise AccountConfig_ProviderTypeNotDefined(data_provider_type= self.data_provider_type)

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 13
@dataclass
class DomoAccount_Config_AbstractCredential(DomoAccount_Config):
    credentials: dict = None
    data_provider_type : str= "abstract-credential-store"
    is_oauth : bool= False

    
    @classmethod
    def _from_json(cls, obj, parent = None, **kwargs):

        return cls(
            parent = parent,
            raw = obj,
            credentials=obj['credentials'],
        )

    def to_json(self):
        return {"credentials": deepcopy(self.credentials)}

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 15
@dataclass
class DomoAccount_Config_DatasetCopy(DomoAccount_Config):
    data_provider_type = "dataset-copy"
    is_oauth : bool = False
    
    domo_instance: str = None
    access_token: str = field(repr=False, default = None)


    @classmethod
    def _from_json(cls, obj : dict  , parent : Any =None, **kwargs):
        
        return cls(
            access_token=obj.get('accessToken'),
            domo_instance=obj.get('instance'),
            parent=parent,
            raw=obj,
            data_provider_type = "dataset-copy"
        )

    def to_json(self):
        return {"accessToken": self.access_token, "instance": self.domo_instance}


@dataclass
class DomoAccount_Config_DomoAccessToken(DomoAccount_Config):

    data_provider_type: str = "domo-access-token"
    is_oauth : bool = False
    
    domo_access_token: str = field(repr=False, default=None)
    username: str = None
    password: str = field(repr=False, default=None)


    
    @classmethod
    def _from_json(cls, obj : dict , parent : Any =None, **kwargs):

        return cls(
            domo_access_token=obj.get("domoAccessToken"),
            username=obj.get("username"),
            password=obj.get("password"),
            parent=parent,
            raw=obj,
            is_oauth = None
        )

    def to_json(self):
        return {
            "domoAccessToken": self.domo_access_token or "",
            "username": self.username or "",
            "password": self.password or "",
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 18
@dataclass
class DomoAccount_Config_Governance(DomoAccount_Config):
    is_oauth: bool = False
    data_provider_type: str = "domo-governance-d14c2fef-49a8-4898-8ddd-f64998005600"

    domo_instance: str = None
    access_token: str = field(repr=False, default=None)

    @classmethod
    def _from_json(cls, obj : dict , parent: Any = None, **kwargs):

        return cls(
            access_token=obj.get("apikey"),
            domo_instance=obj.get("customer"),
            parent=parent,
            raw = obj
        )

    def to_json(self):
        return {"apikey": self.access_token, "customer": self.domo_instance}

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 21
@dataclass
class DomoAccount_Config_AmazonS3(DomoAccount_Config):
    data_provider_type: str  = "amazon-s3"
    is_oauth: bool = False

    access_key: str = None
    secret_key: str = field(repr=False, default = None)
    bucket: str = None
    region: str = "us-west-2"

    @classmethod
    def _from_json(cls, obj : dict ,parent : Any = None,  **kwargs):
        dd = util_dd.DictDot(obj)

        return cls(
            access_key=dd.accessKey,
            secret_key=dd.secretKey,
            bucket=dd.bucket,
            region=dd.region,
            parent = parent,
            raw = obj
        )

    def to_json(self):
        bucket = self.bucket

        if bucket and bucket.lower().startswith("s3://"):
            bucket = bucket[5:]
            print(
                f"🤦‍♀️- Domo bucket expects string without s3:// prefix. Trimming to '{bucket}' for the output"
            )
        return {
            "accessKey": self.access_key,
            "secretKey": self.secret_key,
            "bucket": bucket,
            "region": self.region,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 23
@dataclass
class DomoAccount_Config_AmazonS3Advanced(DomoAccount_Config):
    data_provider_type : str = "amazons3-advanced"
    is_oauth: bool = False

    access_key: str = None
    secret_key: str = field(repr=False, default = None)
    
    bucket: str = None
    region: str = "us-west-2"

    @classmethod
    def _from_json(cls, obj : dict = None, parent : Any = None , **kwargs):
        dd = util_dd.DictDot(obj)

        return cls(
            access_key=dd.accessKey,
            secret_key=dd.secretKey,
            bucket=dd.bucket,
            region=dd.region,
            parent = parent
        )

    def to_json(self):
        bucket = self.bucket

        if bucket and bucket.lower().startswith("s3://"):
            bucket = bucket[5:]
            print(
                f"🤦‍♀️- Domo bucket expects string without s3:// prefix. Trimming to '{bucket}' for the output"
            )
        return {
            "accessKey": self.access_key,
            "secretKey": self.secret_key,
            "bucket": bucket,
            "region": self.region,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 24
@dataclass
class DomoAccount_Config_AwsAthena(DomoAccount_Config):
    data_provider_type :str = "aws-athena"
    is_oauth: bool = False

    access_key: str = None
    secret_key: str = field(repr=False, default = None)
    bucket: str = None
    workgroup: str = None

    region: str = "us-west-2"

    @classmethod
    def _from_json(cls, obj : dict, parent :Any = None, **kwargs):
        dd = util_dd.DictDot(obj)

        return cls(
            access_key=dd.awsAccessKey,
            secret_key=dd.awsSecretKey,
            bucket=dd.s3StagingDir,
            region=dd.region,
            workgroup=dd.workgroup,
            parent = parent,
            raw = obj
        )

    def to_json(self):
        return {
            "awsAccessKey": self.access_key,
            "awsSecretKey": self.secret_key,
            "s3StagingDir": self.bucket,
            "region": self.region,
            "workgroup": self.workgroup,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 26
@dataclass
class DomoAccount_Config_HighBandwidthConnector(DomoAccount_Config):
    """this connector is not enabled by default contact your CSM / AE"""
    
    data_provider_type: str = "amazon-athena-high-bandwidth"
    is_oauth : bool = False

    access_key: str = None
    secret_key: str = field(repr=False, default = None)
    bucket: str = None

    region: str = "us-west-2"
    

    @classmethod
    def _from_json(cls, obj: dict, parent : Any = None,**kwargs):
        
        return cls(
            access_key=obj['awsAccessKey'],
            secret_key=obj['awsSecretKey'],
            bucket=obj['s3StagingDir'],
            region=obj['region'],
            data_provider_type = "amazon-athena-high-bandwidth",
            parent = parent,
            raw = obj
        )

    def to_json(self):
        return {
            "awsAccessKey": self.access_key,
            "awsSecretKey": self.secret_key,
            "s3StagingDir": self.bucket,
            "region": self.region,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 30
@dataclass
class DomoAccount_Config_Snowflake(DomoAccount_Config):
    """this connector is not enabled by default contact your CSM / AE"""

    data_provider_type: str = "snowflake"
    is_oauth :bool = False

    account: str = None
    username: str = None
    password: str = field(repr=False, default = None)
    role: str = None

    @classmethod
    def _from_json(cls, obj: dict, parent : Any = None, **kwargs):
        dd = util_dd.DictDot(obj)

        return cls(
            account=dd.account,
            username=dd.username,
            password=dd.password,
            role=dd.role,
            raw = obj,
            parent = parent
        )

    def to_json(self):
        return {
            "account": self.account,
            "username": self.username,
            "password": self.password,
            "role": self.role,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 32
@dataclass
class DomoAccount_Config_SnowflakeUnload_V2(DomoAccount_Config):
    """this connector is not enabled by default contact your CSM / AE"""

    data_provider_type :str = "snowflake-unload-v2"
    is_oauth: bool = False
    
    account: str = None
    username: str = None
    password: str = field(repr=False, default = None)

    access_key: str = None
    secret_key: str = field(repr=False, default = None)
    region: str = None 
    bucket: str = None

    role: str = None


    @classmethod
    def _from_json(cls, obj: dict , parent : Any = None, **kwargs):
        dd = util_dd.DictDot(obj)

        return cls(
            account=dd.account,
            username=dd.username,
            password=dd.password,
            access_key=dd.accessKey,
            secret_key=dd.secretKey,
            bucket=dd.bucket,
            region=dd.region,
            role=dd.role,
            raw = obj ,
            parent = parent
        )

    def to_json(self):
        return {
            "account": self.account,
            "username": self.username,
            "password": self.password,
            "role": self.role,
            "accessKey": self.access_key,
            "secretKey": self.secret_key,
            "bucket": self.bucket,
            "region": self.region,
        }




# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 34
@dataclass
class DomoAccount_Config_SnowflakeUnloadAdvancedPartition(DomoAccount_Config):
    data_provider_type : str = "snowflake-internal-unload-advanced-partition"
    is_oauth: bool = False

    password: str = field(repr=False, default = None)
    account: str = None
    username: str = None
    role: str = None


    @classmethod
    def _from_json(cls, obj : dict, parent : Any = None, **kwargs):
        return cls(
            password=obj["password"],
            role=obj.get("role"),
            account=obj["account"],
            username=obj["username"],
            raw = obj, parent = parent
        )

    def to_json(self):
        return {
            "password": self.password,
            "role": self.role,
            "account": self.account,
            "username": self.username,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 36
@dataclass
class DomoAccount_Config_SnowflakeWriteback(DomoAccount_Config):
    data_provider_type :str = "snowflake-writeback"
    is_oauth: bool = False

    domo_client_secret: str = field(repr=False, default = None)
    domo_client_id: str =None
    account: str = None
    password: str = field(repr=False, default = None)
    username: str = None


    @classmethod
    def _from_json(cls, obj: dict, parent : Any = None, **kwargs):
        return cls(
            domo_client_secret=obj["domoClientSecret"],
            domo_client_id=obj["domoClientId"],
            account=obj["account"],
            username=obj["username"],
            password=obj["password"],
            raw = obj,
            parent = parent
        )

    def to_json(self):
        return {
            "domoClientSecret": self.domo_client_secret,
            "password": self.password,
            "domoClientId": self.domo_client_id,
            "account": self.account,
            "username": self.username,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 37
@dataclass
class DomoAccount_Config_SnowflakeUnload(DomoAccount_Config):
    data_provider_type : str = "snowflake-unload"
    is_oauth: bool = False

    secret_key: str = field(repr=False, default = None)
    access_key: str = None
    account: str = None
    password: str = field(repr=False, default = None)
    username: str= None
    bucket: str = None


    @classmethod
    def _from_json(cls, obj: dict, parent : Any = None, **kwargs):
        return cls(
            secret_key=obj["secretKey"],
            access_key=obj["accessKey"],
            account=obj["account"],
            username=obj["username"],
            password=obj["password"],
            bucket=obj["bucket"],
            raw  = obj,
            parent = parent
        )

    def to_json(self):
        return {
            "bucket": self.bucket,
            "password": self.password,
            "secretKey": self.secret_key,
            "accessKey": self.access_key,
            "account": self.account,
            "username": self.username,
        }



# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 38
@dataclass
class DomoAccount_Config_SnowflakeFederated(DomoAccount_Config):
    data_provider_type :str = "snowflake-federated"
    is_oauth: bool = False
    password: str = field(repr=False, default = None)

    host: str = None
    warehouse: str = None
    username: str = None
    port: str = None
    role: str = None


    @classmethod
    def _from_json(cls, obj: dict, parent : Any = None, **kwargs):
        return cls(
            password=obj["password"],
            host=obj["host"],
            warehouse=obj["warehouse"],
            username=obj["user"],
            role=obj.get("role"),
            port=obj.get("port"),
            raw = obj,
            parent = parent
        )

    def to_json(self):
        return {
            "password": self.password,
            "port": self.port,
            "host": self.host,
            "warehouse": self.warehouse,
            "user": self.username,
            "role": self.role,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 39
@dataclass
class DomoAccount_Config_SnowflakeInternalUnload(DomoAccount_Config):
    is_oauth: bool = False
    data_provider_type : str = "snowflake-internal-unload"

    password: str = field(repr=False, default = None)
    account: str = None
    username: str= None
    role: str = None

    @classmethod
    def _from_json(cls, obj : dict , parent : Any = None, **kwargs):
        return cls(
            password=obj["password"],
            role=obj.get("role"),
            account=obj["account"],
            username=obj["username"],
            parent = parent,
            raw = obj
        )

    def to_json(self):
        return {
            "password": self.password,
            "role": self.role,
            "account": self.account,
            "username": self.username,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 40
@dataclass
class DomoAccount_Config_SnowflakeKeyPairAuthentication(DomoAccount_Config):
    data_provider_type : str = "snowflakekeypairauthentication"
    is_oauth: bool = False

    private_key: str = field(repr=False , default = None)
    account: str = field(repr=False , default = None)
    passphrase: str = field(repr=False,  default = None)
    username: str = None
    role: str = None

    @classmethod
    def _from_json(cls, obj: dict, parent : Any = None, **kwargs):
        return cls(
            private_key=obj["privateKey"],
            role=obj.get("role"),
            account=obj["account"],
            username=obj["username"],
            passphrase=obj["passPhrase"],
            parent = parent,
            raw = obj
        )

    def to_json(self):
        return {
            "privateKey": self.private_key,
            "role": self.role,
            "account": self.account,
            "username": self.username,
            "passPhrase": self.passphrase,
        }

# %% ../../nbs/classes/50_DomoAccount_Default_Config.ipynb 42
@staticmethod
@patch_to(ConfigEnum)
def generate_alt_search_str(raw_value):
    return raw_value.lower().replace("-", "_")



class AccountConfig(ConfigEnum):
    """
    Enum provides appropriate spelling for data_provider_type and config object.
    The name of the enum should correspond with the data_provider_type with hyphens replaced with underscores.
    """

    abstract_credential_store = DomoAccount_Config_AbstractCredential
    dataset_copy = DomoAccount_Config_DatasetCopy
    domo_access_token = DomoAccount_Config_DomoAccessToken
    domo_governance_d14c2fef_49a8_4898_8ddd_f64998005600 = DomoAccount_Config_Governance
    aws_athena = DomoAccount_Config_AwsAthena
    amazon_athena_high_bandwidth = DomoAccount_Config_HighBandwidthConnector
    amazon_s3 = DomoAccount_Config_AmazonS3
    amazons3_advanced = DomoAccount_Config_AmazonS3Advanced

    snowflake = DomoAccount_Config_Snowflake

    snowflake_unload = DomoAccount_Config_SnowflakeUnload
    snowflake_unload_v2 = DomoAccount_Config_SnowflakeUnload_V2

    snowflake_internal_unload_advanced_partition = (
        DomoAccount_Config_SnowflakeUnloadAdvancedPartition
    )

    snowflake_internal_unload = DomoAccount_Config_SnowflakeInternalUnload

    snowflakekeypairauthentication = DomoAccount_Config_SnowflakeKeyPairAuthentication

    snowflake_writeback = DomoAccount_Config_SnowflakeWriteback
    snowflake_federated = DomoAccount_Config_SnowflakeFederated

    _uses_oauth = ["google_spreadsheets"]

    _config_oauth = DomoAccount_NoConfig_OAuth
    _config_notdefined = DomoAccount_NoConfig


    @classmethod
    def _missing_(cls, value):
        try:
            alt_search_str = cls.generate_alt_search_str(value)
    
            config_match = next(
                (member for member in cls if member.name in [value , alt_search_str]), None
            )

            ## best case scenario alt_search yields a result
            if config_match:
                return config_match

            ## second best case, display_type is an oauth and therefore has mo matching config
            oauth_match = next(
                (
                    oauth_str
                    for oauth_str in cls._uses_oauth.value
                    if oauth_str in [ value , alt_search_str ]
                ),
                None,
            )
            if oauth_match:
                raise AccountConfig_UsesOauth(value)

            ## worst case, unencountered display_type
            raise AccountConfig_ProviderTypeNotDefined(value)

        except AccountConfig_UsesOauth as e:
            print(e)
            return cls._config_oauth

        except AccountConfig_ProviderTypeNotDefined as e:
            print(e)
            return cls._config_notdefined

